@@ -30,7 +30,6 @@
 #include "xf86.h"
 #include "xf86_OSproc.h"
 #include "xf86Resources.h"
-#include "xf86_ansic.h"
 #include "compiler.h"
 #include "xf86PciInfo.h"
 #include "xf86Pci.h"
@@ -671,7 +670,11 @@ TRIDENTDisplayVideo(
     OUTW(vgaIOBase + 4, ((width<<1) & 0xff00)      | 0x91);
     OUTW(vgaIOBase + 4, ((offset) & 0xff) << 8     | 0x92);
     OUTW(vgaIOBase + 4, ((offset) & 0xff00)        | 0x93);
-    OUTW(vgaIOBase + 4, ((offset) & 0x070000) >> 8 | 0x94);
+    if (pTrident->Chipset >= CYBER9397) {
+    	OUTW(vgaIOBase + 4, ((offset) & 0x0f0000) >> 8 | 0x94);
+    } else {
+    	OUTW(vgaIOBase + 4, ((offset) & 0x070000) >> 8 | 0x94);
+    }
     
     /* Horizontal Zoom */
     if (pTrident->videoFlags & VID_ZOOM_INV) {
@@ -1183,6 +1186,7 @@ tridentFixFrame(ScrnInfoPtr pScrn, int *fixFrame)
     int h_off = 0;
     int v_off = 0;
     unsigned char CRTC[0x11];
+    unsigned char hcenter, vcenter;
     Bool isShadow;
     unsigned char shadow = 0;
 
@@ -1209,6 +1213,10 @@ tridentFixFrame(ScrnInfoPtr pScrn, int *fixFrame)
     CRTC[0x7] = INB(vgaIOBase + 5);
     OUTB(vgaIOBase + 4, 0x10);
     CRTC[0x10] = INB(vgaIOBase + 5);
+    OUTB(0x3CE, HorStretch);
+    hcenter = INB(0x3CF);
+    OUTB(0x3CE, VertStretch);
+    vcenter = INB(0x3CF);
 
     HTotal = CRTC[0] << 3;
     VTotal = CRTC[6] 
@@ -1223,15 +1231,23 @@ tridentFixFrame(ScrnInfoPtr pScrn, int *fixFrame)
     if (isShadow) {
 	SHADOW_RESTORE(shadow);
 	if (pTrident->lcdMode != 0xff) {
+	    if (hcenter & 0x80) {
 	    h_off = (LCD[pTrident->lcdMode].display_x 
 		     - pScrn->currentMode->HDisplay) >> 1;
+		switch (pTrident->Chipset) {
+		    case BLADEXP:
+			h_off -= 5;
+		}
+	    }
+	    if (vcenter & 0x80) {
 	    v_off = (LCD[pTrident->lcdMode].display_y 
 		     - pScrn->currentMode->VDisplay) >> 1;
 	}
     } 
+    } 
 
-    pTrident->hsync = (HTotal - HSyncStart) + 23 + h_off;
-    pTrident->vsync = (VTotal - VSyncStart) - 2 + v_off;
+    pTrident->hsync = HTotal - HSyncStart + 23 + h_off;
+    pTrident->vsync = VTotal - VSyncStart - 2 + v_off;
     pTrident->hsync_rskew = 0;
     pTrident->vsync_bskew = 0;
   
